@@ -472,6 +472,7 @@ impl<I, P> PriorityQueue<I, P>
     /// Internal function that transform the `heap`
     /// vector in a heap with its properties
     fn heap_build(&mut self){
+        if self.size == 0 {return;}
         for i in (0..parent(self.size)).rev(){
             self.heapify(i);
         }
@@ -641,3 +642,78 @@ fn log2_fast(x: usize) -> usize {
 fn better_to_rebuild(len1: usize, len2: usize) -> bool {
     2 * (len1 + len2) < len2 * log2_fast(len1)
 }
+
+#[cfg(serde)]
+mod serde {
+    use pqueue::PriorityQueue;
+
+    use std::hash::Hash;
+    use std::cmp::{Ord, Eq};
+    use std::marker::PhantomData;
+
+    extern crate serde;
+    use self::serde::ser::{Serialize, Serializer, SerializeMap};
+    impl<I, P> Serialize for PriorityQueue<I, P>
+        where I: Hash + Eq + Serialize,
+              P: Ord + Serialize {
+        fn serialize<S> (&self, serializer: S) -> Result<S::Ok, S::Error>
+            where S:Serializer {
+            let mut map_serializer = serializer.serialize_map(Some(self.size))?;
+            for (k, v) in &self.map {
+                map_serializer.serialize_key(k)?;
+                map_serializer.serialize_value(v.as_ref().unwrap())?;
+            }
+            map_serializer.end()
+        }
+    }
+
+    use self::serde::de::{Deserialize, Deserializer, Visitor, MapAccess};
+    impl<'de, I, P> Deserialize<'de> for PriorityQueue<I, P>
+        where I: Hash + Eq + Deserialize<'de>,
+              P: Ord + Deserialize<'de> {
+        fn deserialize<D>(deserializer: D) -> Result<PriorityQueue<I, P>, D::Error>
+            where D: Deserializer<'de> {
+            deserializer.deserialize_map(PQVisitor{marker: PhantomData})
+        }
+    }
+
+    struct PQVisitor<I, P>
+        where I: Hash + Eq,
+              P: Ord {
+        marker: PhantomData<PriorityQueue<I, P>>
+    }
+    impl<'de, I, P> Visitor<'de> for PQVisitor<I, P>
+        where I: Hash + Eq + Deserialize<'de>,
+              P: Ord + Deserialize<'de> {
+        type Value = PriorityQueue<I, P>;
+
+        fn expecting(&self, formatter: &mut ::std::fmt::Formatter) -> ::std::fmt::Result {
+            write!(formatter, "A priority queue")
+        }
+
+        fn visit_unit<E>(self) -> Result<Self::Value, E> {
+            Ok(PriorityQueue::new())
+        }
+
+        fn visit_map<A>(self, mut map: A) -> Result<Self::Value, A::Error>
+            where A: MapAccess<'de>{
+            let mut pq: PriorityQueue<I, P> = 
+                if let Some(size) = map.size_hint() {
+                    PriorityQueue::with_capacity(size)
+                } else {
+                    PriorityQueue::new()
+                };
+
+            while let Some((item, priority)) = map.next_entry()? {
+                pq.map.insert(item, Some(priority));
+                pq.qp.push(pq.size);
+                pq.heap.push(pq.size);
+                pq.size+=1;
+            }
+            pq.heap_build();
+            // if it is guaranteed that deserialization follow the same order of
+            // serialization, heap_build is useless, but anyway should be O(n)
+            Ok(pq)
+        }
+    }
+}
